@@ -1,21 +1,20 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public License
- *  as published by the Free Software Foundation; either version 2.1
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
- *  02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
 
@@ -62,7 +61,7 @@ g_mime_message_part_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeMessagePart),
-			0,    /* n_preallocs */
+			16,   /* n_preallocs */
 			(GInstanceInitFunc) g_mime_message_part_init,
 		};
 		
@@ -223,7 +222,7 @@ g_mime_message_part_new (const char *subtype)
 	GMimeMessagePart *part;
 	GMimeContentType *type;
 	
-	part = g_object_new (GMIME_TYPE_MESSAGE_PART, NULL);
+	part = g_object_new (GMIME_TYPE_MESSAGE_PART, NULL, NULL);
 	
 	type = g_mime_content_type_new ("message", subtype ? subtype : "rfc822");
 	g_mime_object_set_content_type (GMIME_OBJECT (part), type);
