@@ -1,21 +1,20 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public License
- *  as published by the Free Software Foundation; either version 2.1
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
- *  02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
 
@@ -31,17 +30,6 @@
 #include "gmime-stream-mem.h"
 #include "gmime-parser.h"
 
-
-/**
- * SECTION: gmime-message-partial
- * @title: GMimeMessagePartial
- * @short_description: Partial MIME parts
- * @se_also:
- *
- * A #GMimeMessagePartial represents the message/partial MIME part.
- **/
-
-
 /* GObject class methods */
 static void g_mime_message_partial_class_init (GMimeMessagePartialClass *klass);
 static void g_mime_message_partial_init (GMimeMessagePartial *catpart, GMimeMessagePartialClass *klass);
@@ -162,10 +150,10 @@ message_partial_set_content_type (GMimeObject *object, GMimeContentType *content
 	partial->id = g_strdup (value);
 	
 	value = g_mime_content_type_get_parameter (content_type, "number");
-	partial->number = value ? strtol (value, NULL, 10) : -1;
+	partial->number = value ? atoi (value) : -1;
 	
 	value = g_mime_content_type_get_parameter (content_type, "total");
-	partial->total = value ? strtol (value, NULL, 10) : -1;
+	partial->total = value ? atoi (value) : -1;
 	
 	GMIME_OBJECT_CLASS (parent_class)->set_content_type (object, content_type);
 }
@@ -188,7 +176,7 @@ g_mime_message_partial_new (const char *id, int number, int total)
 	GMimeContentType *type;
 	char *num;
 	
-	partial = g_object_new (GMIME_TYPE_MESSAGE_PARTIAL, NULL);
+	partial = g_object_new (GMIME_TYPE_MESSAGE_PARTIAL, NULL, NULL);
 	
 	type = g_mime_content_type_new ("message", "partial");
 	
@@ -234,7 +222,7 @@ g_mime_message_partial_get_id (GMimeMessagePartial *partial)
  *
  * Gets the message/partial part number.
  *
- * Returns the message/partial part number or %-1 on fail.
+ * Returns the message/partial part number or -1 on fail.
  **/
 int
 g_mime_message_partial_get_number (GMimeMessagePartial *partial)
@@ -302,11 +290,13 @@ g_mime_message_partial_reconstruct_message (GMimeMessagePartial **partials, size
 	
 	g_return_val_if_fail (num > 0, NULL);
 	
-	if (!(id = g_mime_message_partial_get_id (partials[0])))
+	id = g_mime_message_partial_get_id (partials[0]);
+	if (!id)
 		return NULL;
 	
 	/* get them into the correct order... */
-	qsort ((void *) partials, num, sizeof (gpointer), partial_compare);
+	qsort ((void *) partials, num, sizeof (gpointer),
+	       partial_compare);
 	
 	/* only the last message/partial part is REQUIRED to have the total parameter */
 	total = g_mime_message_partial_get_total (partials[num - 1]);
@@ -410,6 +400,8 @@ g_mime_message_partial_split_message (GMimeMessage *message, size_t max_size, si
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
 	stream = g_mime_stream_mem_new ();
+	buf = ((GMimeStreamMem *) stream)->buffer->data;
+	
 	if (g_mime_object_write_to_stream (GMIME_OBJECT (message), stream) == -1) {
 		g_object_unref (stream);
 		return NULL;
@@ -433,8 +425,6 @@ g_mime_message_partial_split_message (GMimeMessage *message, size_t max_size, si
 	
 	start = 0;
 	parts = g_ptr_array_new ();
-	buf = ((GMimeStreamMem *) stream)->buffer->data;
-	
 	while (start < len) {
 		/* Preferably, we'd split on whole-lines if we can,
 		 * but if that's not possible, split on max size */
