@@ -1,21 +1,20 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public License
- *  as published by the Free Software Foundation; either version 2.1
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
- *  02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
 
@@ -27,15 +26,19 @@
 #include <string.h>
 #include <sys/types.h>
 #include <regex.h>
+#include <ctype.h>
 
 #include "gmime-parser.h"
 
-#include "gmime-table-private.h"
 #include "gmime-stream-mem.h"
 #include "gmime-message-part.h"
 #include "gmime-multipart.h"
 #include "gmime-part.h"
 
+#ifndef HAVE_ISBLANK
+#define isblank(c) ((c) == ' ' || (c) == '\t')
+#endif /* HAVE_ISBLANK */
+
 #ifdef ENABLE_WARNINGS
 #define w(x) x
 #else
@@ -44,19 +47,6 @@
 
 #define d(x)
 
-
-/**
- * SECTION: gmime-parser
- * @title: GMimeParser
- * @short_description: Message and MIME part parser
- * @see_also:
- *
- * A #GMimeParser parses a stream into a #GMimeMessage or other
- * #GMimeObject and can also handle parsing MBox formatted streams
- * into multiple #GMimeMessage objects.
- **/
-
-
 static void g_mime_parser_class_init (GMimeParserClass *klass);
 static void g_mime_parser_init (GMimeParser *parser, GMimeParserClass *klass);
 static void g_mime_parser_finalize (GObject *object);
@@ -81,19 +71,20 @@ enum {
 	GMIME_PARSER_STATE_HEADERS,
 	GMIME_PARSER_STATE_HEADERS_END,
 	GMIME_PARSER_STATE_CONTENT,
-	GMIME_PARSER_STATE_COMPLETE,
 };
 
 struct _GMimeParserPrivate {
+	int state;
+	
 	GMimeStream *stream;
 	
 	off_t offset;
 	
 	/* i/o buffers */
-	char realbuf[SCAN_HEAD + SCAN_BUF + 1];
-	char *inbuf;
-	char *inptr;
-	char *inend;
+	unsigned char realbuf[SCAN_HEAD + SCAN_BUF + 1];
+	unsigned char *inbuf;
+	unsigned char *inptr;
+	unsigned char *inend;
 	
 	off_t from_offset;
 	GByteArray *from_line;
@@ -103,32 +94,27 @@ struct _GMimeParserPrivate {
 	gpointer user_data;
 	
 	/* header buffer */
-	char *headerbuf;
-	char *headerptr;
-	size_t headerleft;
+	unsigned char *headerbuf;
+	unsigned char *headerptr;
+	unsigned int headerleft;
 	
 	/* raw header buffer */
-	char *rawbuf;
-	char *rawptr;
-	size_t rawleft;
+	unsigned char *rawbuf;
+	unsigned char *rawptr;
+	unsigned int rawleft;
 	
-	/* current message headerblock offsets */
-	off_t headers_begin;
-	off_t headers_end;
+	off_t headers_start;
+	off_t header_start;
 	
-	/* current header field offset */
-	off_t header_offset;
-	
-	short int state;
-	
-	unsigned short int unused:10;
-	unsigned short int midline:1;
-	unsigned short int seekable:1;
-	unsigned short int scan_from:1;
-	unsigned short int have_regex:1;
-	unsigned short int persist_stream:1;
-	unsigned short int respect_content_length:1;
+	unsigned int unstep:26;
+	unsigned int midline:1;
+	unsigned int seekable:1;
+	unsigned int scan_from:1;
+	unsigned int have_regex:1;
+	unsigned int persist_stream:1;
+	unsigned int respect_content_length:1;
 	
+	GMimeContentType *content_type;
 	struct _header_raw *headers;
 	
 	struct _boundary_stack *bounds;
@@ -136,7 +122,7 @@ struct _GMimeParserPrivate {
 
 struct _boundary_stack {
 	struct _boundary_stack *parent;
-	char *boundary;
+	unsigned char *boundary;
 	size_t boundarylen;
 	size_t boundarylenfinal;
 	size_t boundarylenmax;
@@ -247,7 +233,7 @@ g_mime_parser_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeParser),
-			0,    /* n_preallocs */
+			16,   /* n_preallocs */
 			(GInstanceInitFunc) g_mime_parser_init,
 		};
 		
@@ -328,11 +314,10 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	priv->rawptr = priv->rawbuf;
 	priv->rawleft = SCAN_HEAD;
 	
-	priv->headers_begin = -1;
-	priv->headers_end = -1;
-	
-	priv->header_offset = -1;
+	priv->headers_start = -1;
+	priv->header_start = -1;
 	
+	priv->unstep = 0;
 	priv->midline = FALSE;
 	priv->seekable = offset != -1;
 	
@@ -360,6 +345,14 @@ parser_close (GMimeParser *parser)
 		parser_pop_boundary (parser);
 }
 
+static void
+parser_unstep (GMimeParser *parser)
+{
+	struct _GMimeParserPrivate *priv = parser->priv;
+	
+	priv->unstep++;
+}
+
 
 /**
  * g_mime_parser_new:
@@ -590,7 +583,7 @@ static ssize_t
 parser_fill (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	char *inbuf, *inptr, *inend;
+	unsigned char *inbuf, *inptr, *inend;
 	ssize_t nread;
 	size_t inlen;
 	
@@ -633,7 +626,7 @@ parser_fill (GMimeParser *parser)
 
 
 static off_t
-parser_offset (struct _GMimeParserPrivate *priv, const char *inptr)
+parser_offset (struct _GMimeParserPrivate *priv, unsigned char *inptr)
 {
 	if (priv->offset == -1)
 		return -1;
@@ -687,10 +680,9 @@ static int
 parser_step_from (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	register char *inptr;
-	char *start, *inend;
-	ssize_t left = 0;
-	size_t len;
+	register unsigned char *inptr;
+	unsigned char *start, *inend;
+	size_t len, left = 0;
 	
 	g_byte_array_set_size (priv->from_line, 0);
 	
@@ -728,7 +720,7 @@ parser_step_from (GMimeParser *parser)
 			
 			if (len >= 5 && !strncmp (start, "From ", 5)) {
 				priv->from_offset = parser_offset (priv, start);
-				g_byte_array_append (priv->from_line, (unsigned char *) start, len);
+				g_byte_array_append (priv->from_line, start, len);
 				goto got_from;
 			}
 		}
@@ -746,31 +738,19 @@ parser_step_from (GMimeParser *parser)
 	return 0;
 }
 
-static inline size_t
-nearest_pow (size_t num)
-{
-	size_t n = num > 0 ? num - 1 : 0;
-	
-	n |= n >> 1;
-	n |= n >> 2;
-	n |= n >> 4;
-	n |= n >> 8;
-	n |= n >> 16;
-	n++;
-	
-	return n;
-}
-
 #define header_append(priv, start, len) G_STMT_START {                    \
 	if (priv->headerleft <= len) {                                    \
 		size_t hlen, hoff;                                        \
 		                                                          \
-		hoff = priv->headerptr - priv->headerbuf;                 \
-		hlen = nearest_pow (hoff + len + 1);                      \
+		hlen = hoff = priv->headerptr - priv->headerbuf;          \
+		hlen = hlen ? hlen : 1;                                   \
 		                                                          \
-		priv->headerbuf = g_realloc (priv->headerbuf, hlen);      \
+		while (hlen < hoff + len)                                 \
+			hlen <<= 1;                                       \
+		                                                          \
+		priv->headerbuf = g_realloc (priv->headerbuf, hlen + 1);  \
 		priv->headerptr = priv->headerbuf + hoff;                 \
-		priv->headerleft = (hlen - 1) - hoff;                     \
+		priv->headerleft = hlen - hoff;                           \
 	}                                                                 \
 	                                                                  \
 	memcpy (priv->headerptr, start, len);                             \
@@ -782,12 +762,15 @@ nearest_pow (size_t num)
 	if (priv->rawleft <= len) {                                       \
 		size_t hlen, hoff;                                        \
 		                                                          \
-		hoff = priv->rawptr - priv->rawbuf;                       \
-		hlen = nearest_pow (hoff + len + 1);                      \
+		hlen = hoff = priv->rawptr - priv->rawbuf;                \
+		hlen = hlen ? hlen : 1;                                   \
 		                                                          \
-		priv->rawbuf = g_realloc (priv->rawbuf, hlen);            \
+		while (hlen < hoff + len)                                 \
+			hlen <<= 1;                                       \
+		                                                          \
+		priv->rawbuf = g_realloc (priv->rawbuf, hlen + 1);        \
 		priv->rawptr = priv->rawbuf + hoff;                       \
-		priv->rawleft = (hlen - 1) - hoff;                        \
+		priv->rawleft = hlen - hoff;                              \
 	}                                                                 \
 	                                                                  \
 	memcpy (priv->rawptr, start, len);                                \
@@ -800,89 +783,58 @@ nearest_pow (size_t num)
 	priv->rawptr = priv->rawbuf;                                      \
 } G_STMT_END
 
-static void
-header_parse (GMimeParser *parser, struct _header_raw **tail)
-{
-	struct _GMimeParserPrivate *priv = parser->priv;
-	struct _header_raw *header;
-	register char *inptr, *end;
-	char *start;
-	size_t hlen;
-	
-	*priv->headerptr = '\0';
-	inptr = priv->headerbuf;
-	while (*inptr && *inptr != ':' && !is_type (*inptr, IS_SPACE | IS_CTRL))
-		inptr++;
-	
-	if (*inptr != ':') {
-		/* ignore invalid headers */
-		w(g_warning ("Invalid header at %lld: '%s'",
-			     (long long) priv->header_offset,
-			     priv->headerbuf));
-		
-		priv->headerleft += priv->headerptr - priv->headerbuf;
-		priv->headerptr = priv->headerbuf;
-		
-		return;
-	}
-	
-	hlen = inptr - priv->headerbuf;
-	
-	header = g_new (struct _header_raw, 1);
-	header->next = NULL;
-	
-	header->name = g_strndup (priv->headerbuf, hlen);
-	
-	/* skip over leading lwsp */
-	inptr++;
-	while (is_lwsp (*inptr))
-		inptr++;
-	
-	/* cut trailing lwsp */
-	start = inptr++;
-	end = inptr;
-	
-	while (*inptr) {
-		if (!is_lwsp (*inptr++))
-			end = inptr;
-	}
-	
-	header->value = g_strndup (start, end - start);
-	
-	header->offset = priv->header_offset;
-	
-	(*tail)->next = header;
-	*tail = header;
-	
-	priv->headerleft += priv->headerptr - priv->headerbuf;
-	priv->headerptr = priv->headerbuf;
-	
-	if (priv->have_regex &&
-	    !regexec (&priv->header_regex, header->name, 0, NULL, 0))
-		priv->header_cb (parser, header->name, header->value,
-				 header->offset, priv->user_data);
-}
+#define header_parse(parser, priv, hend) G_STMT_START {                   \
+	register unsigned char *colon;                                    \
+	struct _header_raw *header;                                       \
+	size_t hlen;                                                      \
+	                                                                  \
+	header = g_new (struct _header_raw, 1);                           \
+	header->next = NULL;                                              \
+	                                                                  \
+	*priv->headerptr = '\0';                                          \
+	colon = priv->headerbuf;                                          \
+	while (*colon && *colon != ':')                                   \
+		colon++;                                                  \
+	                                                                  \
+	hlen = colon - priv->headerbuf;                                   \
+	                                                                  \
+	header->name = g_strndup (priv->headerbuf, hlen);                 \
+	g_strstrip (header->name);                                        \
+	if (*colon != ':') {                                              \
+		w(g_warning ("Invalid header: %s", header->name));        \
+		header->value = header->name;                             \
+		header->name = g_strdup ("X-Invalid-Header");             \
+	} else {                                                          \
+		header->value = g_strdup (colon + 1);                     \
+		g_strstrip (header->value);                               \
+	}                                                                 \
+	header->offset = priv->header_start;                              \
+	                                                                  \
+	hend->next = header;                                              \
+	hend = header;                                                    \
+	                                                                  \
+	priv->headerleft += priv->headerptr - priv->headerbuf;            \
+	priv->headerptr = priv->headerbuf;                                \
+	                                                                  \
+	if (priv->have_regex &&                                           \
+	    !regexec (&priv->header_regex, header->name, 0, NULL, 0))     \
+		priv->header_cb (parser, header->name, header->value,     \
+				 header->offset, priv->user_data);        \
+} G_STMT_END
 
 static int
 parser_step_headers (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	gboolean eoln, valid = TRUE, fieldname = TRUE;
-	gboolean continuation = FALSE;
+	register unsigned char *inptr;
+	unsigned char *start, *inend;
 	struct _header_raw *hend;
-	register char *inptr;
-	char *start, *inend;
-	ssize_t left = 0;
-	size_t len;
+	size_t len, left = 0;
 	
 	priv->midline = FALSE;
-	raw_header_reset (priv);
-	header_raw_clear (&priv->headers);
 	hend = (struct _header_raw *) &priv->headers;
-	priv->header_offset = parser_offset (priv, NULL);
-	
-	if (priv->headers_begin == -1)
-		priv->headers_begin = parser_offset (priv, NULL);
+	priv->headers_start = parser_offset (priv, NULL);
+	priv->header_start = parser_offset (priv, NULL);
 	
 	inptr = priv->inptr;
 	inend = priv->inend;
@@ -901,100 +853,43 @@ parser_step_headers (GMimeParser *parser)
 		
 		while (inptr < inend) {
 			start = inptr;
-			
-			/* if we are scanning a new line, check for a folded header */
-			if (!priv->midline && continuation && (*inptr != ' ' && *inptr != '\t')) {
-				header_parse (parser, &hend);
-				priv->header_offset = parser_offset (priv, inptr);
-				continuation = FALSE;
-				fieldname = TRUE;
-				valid = TRUE;
-			}
-			
-			eoln = inptr[0] == '\n' || (inptr[0] == '\r' && inptr[1] == '\n');
-			if (fieldname && !eoln) {
-				/* scan and validate the field name */
-				if (*inptr != ':') {
-					/* Note: see optimization comment [1] */
-					*inend = ':';
-					
-					while (*inptr != ':') {
-						if (is_type (*inptr, IS_SPACE | IS_CTRL)) {
-							valid = FALSE;
-							break;
-						}
-						
-						inptr++;
-					}
-					
-					if (inptr == inend) {
-						/* don't have the full field name */
-						left = inend - start;
-						priv->inptr = start;
-						goto refill;
-					}
-					
-					/* Note: see optimization comment [1] */
-					*inend = '\n';
-				} else if (*inptr == ':') {
-					valid = FALSE;
-				}
-				
-				if (!valid) {
-					if (priv->scan_from && (inptr - start) == 4
-					    && !strncmp (start, "From ", 5))
-						goto next_message;
-					
-					if (priv->headers != NULL || *inptr == ':') {
-						/* probably the start of the content,
-						 * a broken mailer didn't terminate the
-						 * headers with an empty line. *sigh* */
-						goto content_start;
-					}
-				}
-			}
-			
-			fieldname = FALSE;
-			
 			/* Note: see optimization comment [1] */
 			while (*inptr != '\n')
 				inptr++;
 			
-			len = (size_t) (inptr - start);
+			raw_header_append (priv, start, inptr - start);
 			
 			if (inptr == inend) {
-				/* we don't have the full line, save
-				 * what we have and refill our
-				 * buffer... */
-				if (inptr > start && inptr[-1] == '\r') {
-					inptr--;
-					len--;
-				}
-				
-				raw_header_append (priv, start, len);
-				header_append (priv, start, len);
+				/* we don't have enough data to tell if we
+				   got all of the header or not... */
+				header_append (priv, start, inptr - start);
 				priv->midline = TRUE;
 				left = inend - inptr;
 				priv->inptr = inptr;
 				goto refill;
 			}
 			
-			raw_header_append (priv, start, len);
+			/* check to see if we've reached the end of the headers */
+			if (!priv->midline && (inptr == start || (inptr - start == 1 && *start == '\r')))
+				goto headers_end;
 			
+			len = inptr - start;
 			if (inptr > start && inptr[-1] == '\r')
 				len--;
 			
-			/* check to see if we've reached the end of the headers */
-			if (!priv->midline && len == 0)
-				goto headers_end;
-			
 			header_append (priv, start, len);
 			
 			/* inptr has to be less than inend - 1 */
-			raw_header_append (priv, "\n", 1);
-			priv->midline = FALSE;
-			continuation = TRUE;
+			raw_header_append (priv, inptr, 1);
 			inptr++;
+			
+			if (*inptr == ' ' || *inptr == '\t') {
+				priv->midline = TRUE;
+			} else {
+				priv->midline = FALSE;
+				header_parse (parser, priv, hend);
+				priv->header_start = parser_offset (priv, inptr);
+			}
 		}
 		
 		left = inend - inptr;
@@ -1004,44 +899,22 @@ parser_step_headers (GMimeParser *parser)
 	inptr = priv->inptr;
 	inend = priv->inend;
 	
-	len = (size_t) (inend - inptr);
-	header_append (priv, inptr, len);
-	raw_header_append (priv, inptr, len);
+	header_append (priv, inptr, inend - inptr);
+	raw_header_append (priv, inptr, inend - inptr);
 	
  headers_end:
 	
 	if (priv->headerptr > priv->headerbuf)
-		header_parse (parser, &hend);
+		header_parse (parser, priv, hend);
 	
-	if (priv->headers_end == -1)
-		priv->headers_end = parser_offset (priv, start);
+	*priv->rawptr = '\0';
 	
 	priv->state = GMIME_PARSER_STATE_HEADERS_END;
-	*priv->rawptr = '\0';
+	
+	g_assert (inptr <= priv->inend);
+	
 	priv->inptr = inptr;
 	
-	return 0;
-	
- next_message:
-	
-	if (priv->headers_end == -1)
-		priv->headers_end = parser_offset (priv, start);
-	
-	priv->state = GMIME_PARSER_STATE_COMPLETE;
-	*priv->rawptr = '\0';
-	priv->inptr = start;
-	
-	return 0;
-	
- content_start:
-	
-	if (priv->headers_end == -1)
-		priv->headers_end = parser_offset (priv, start);
-	
-	priv->state = GMIME_PARSER_STATE_CONTENT;
-	*priv->rawptr = '\0';
-	priv->inptr = start;
-	
 	return 0;
 }
 
@@ -1058,14 +931,54 @@ parser_content_type (GMimeParser *parser)
 }
 
 static int
+parser_step (GMimeParser *parser)
+{
+	struct _GMimeParserPrivate *priv = parser->priv;
+	
+	if (!priv->unstep) {
+	step:
+		switch (priv->state) {
+		case GMIME_PARSER_STATE_INIT:
+			if (priv->scan_from)
+				priv->state = GMIME_PARSER_STATE_FROM;
+			else
+				priv->state = GMIME_PARSER_STATE_HEADERS;
+			goto step;
+			break;
+		case GMIME_PARSER_STATE_FROM:
+			parser_step_from (parser);
+			break;
+		case GMIME_PARSER_STATE_HEADERS:
+			parser_step_headers (parser);
+			break;
+		case GMIME_PARSER_STATE_ERROR:
+			break;
+		default:
+			g_assert_not_reached ();
+			break;
+		}
+	} else {
+		priv->unstep--;
+	}
+	
+	return priv->state;
+}
+
+static void
 parser_skip_line (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	register char *inptr;
-	char *inend;
-	int rv = 0;
+	register unsigned char *inptr;
+	unsigned char *inend;
+	
+	inptr = priv->inptr;
 	
 	do {
+		if (parser_fill (parser) <= 0) {
+			inptr = priv->inptr;
+			break;
+		}
+		
 		inptr = priv->inptr;
 		inend = priv->inend;
 		*inend = '\n';
@@ -1077,74 +990,22 @@ parser_skip_line (GMimeParser *parser)
 			break;
 		
 		priv->inptr = inptr;
-		
-		if (parser_fill (parser) <= 0) {
-			inptr = priv->inptr;
-			rv = -1;
-			break;
-		}
 	} while (1);
 	
 	priv->midline = FALSE;
 	
 	priv->inptr = MIN (inptr + 1, priv->inend);
-	
-	return rv;
 }
 
-static int
-parser_step (GMimeParser *parser)
-{
-	struct _GMimeParserPrivate *priv = parser->priv;
-	
-	switch (priv->state) {
-	case GMIME_PARSER_STATE_ERROR:
-		break;
-	case GMIME_PARSER_STATE_INIT:
-		parser->priv->headers_begin = -1;
-		parser->priv->headers_end = -1;
-		if (priv->scan_from)
-			priv->state = GMIME_PARSER_STATE_FROM;
-		else
-			priv->state = GMIME_PARSER_STATE_HEADERS;
-		break;
-	case GMIME_PARSER_STATE_FROM:
-		parser->priv->headers_begin = -1;
-		parser->priv->headers_end = -1;
-		parser_step_from (parser);
-		break;
-	case GMIME_PARSER_STATE_HEADERS:
-		parser_step_headers (parser);
-		break;
-	case GMIME_PARSER_STATE_HEADERS_END:
-		if (parser_skip_line (parser) == -1)
-			priv->state = GMIME_PARSER_STATE_ERROR;
-		else
-			priv->state = GMIME_PARSER_STATE_CONTENT;
-		break;
-	case GMIME_PARSER_STATE_CONTENT:
-		break;
-	case GMIME_PARSER_STATE_COMPLETE:
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-	
-	return priv->state;
-}
-
-
 enum {
-	FOUND_NOTHING,
-	FOUND_EOS,
+	FOUND_EOS          = 1,
 	FOUND_BOUNDARY,
 	FOUND_END_BOUNDARY
 };
 
-#define content_save(content, start, len) G_STMT_START {                     \
-	if (content)                                                         \
-		g_byte_array_append (content, (unsigned char *) start, len); \
+#define content_save(content, start, len) G_STMT_START {   \
+	if (content)                                       \
+		g_byte_array_append (content, start, len); \
 } G_STMT_END
 
 #define possible_boundary(scan_from, start, len)                                      \
@@ -1152,9 +1013,9 @@ enum {
 			  (len >= 2 && (start[0] == '-' && start[1] == '-')))
 
 static int
-check_boundary (struct _GMimeParserPrivate *priv, const char *start, size_t len)
+check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, size_t len)
 {
-	off_t offset = parser_offset (priv, start);
+	off_t offset = parser_offset (priv, (unsigned char *) start);
 	
 	if (start[len - 1] == '\r')
 		len--;
@@ -1206,8 +1067,8 @@ static int
 parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	register char *inptr;
-	char *start, *inend;
+	register unsigned char *inptr;
+	unsigned char *start, *inend;
 	size_t nleft, len;
 	int found = 0;
 	
@@ -1233,8 +1094,7 @@ parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 		/* Note: see optimization comment [1] */
 		*inend = '\n';
 		
-		len = (size_t) (inend - inptr);
-		if (priv->midline && len == nleft)
+		if (priv->midline && inend - inptr == nleft)
 			found = FOUND_EOS;
 		
 		priv->midline = FALSE;
@@ -1245,7 +1105,7 @@ parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 			while (*inptr != '\n')
 				inptr++;
 			
-			len = (size_t) (inptr - start);
+			len = inptr - start;
 			
 			if (inptr < inend) {
 				if ((found = check_boundary (priv, start, len)))
@@ -1264,7 +1124,7 @@ parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 					goto refill;
 				}
 				
-				/* check for a boundary not ending in a \n (EOF) */
+				/* check for a boundary not ending in a \n */
 				if ((found = check_boundary (priv, start, len)))
 					goto boundary;
 			}
@@ -1303,8 +1163,6 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	off_t start, end;
 	int crlf;
 	
-	g_assert (priv->state >= GMIME_PARSER_STATE_HEADERS_END);
-	
 	if (priv->persist_stream && priv->seekable)
 		start = parser_offset (priv, NULL);
 	else
@@ -1315,20 +1173,19 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 		/* last '\n' belongs to the boundary */
 		if (priv->persist_stream && priv->seekable)
 			end = parser_offset (priv, NULL) - crlf;
-		else if (content->len > crlf)
-			g_byte_array_set_size (content, content->len - crlf);
 		else
-			g_byte_array_set_size (content, 0);
+			g_byte_array_set_size (content, MAX (content->len - crlf, 0));
 	} else if (priv->persist_stream && priv->seekable) {
 		end = parser_offset (priv, NULL);
 	}
 	
 	encoding = g_mime_part_get_encoding (mime_part);
 	
-	if (priv->persist_stream && priv->seekable)
+	if (priv->persist_stream && priv->seekable) {
 		stream = g_mime_stream_substream (priv->stream, start, end);
-	else
+	} else {
 		stream = g_mime_stream_mem_new_with_byte_array (content);
+	}
 	
 	wrapper = g_mime_data_wrapper_new_with_stream (stream, encoding);
 	g_mime_part_set_content_object (mime_part, wrapper);
@@ -1345,16 +1202,10 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 	GMimeMessage *message;
 	GMimeObject *object;
 	
-	g_assert (priv->state == GMIME_PARSER_STATE_CONTENT);
-	
 	/* get the headers */
-	priv->state = GMIME_PARSER_STATE_HEADERS;
-	if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
-		/* Note: currently cannot happen because
-		 * parser_step_headers() never returns error */
-		*found = FOUND_EOS;
-		return;
-	}
+	parser->priv->state = GMIME_PARSER_STATE_HEADERS;
+	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
+		;
 	
 	message = g_mime_message_new (FALSE);
 	header = priv->headers;
@@ -1366,10 +1217,12 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 	if (!(content_type = parser_content_type (parser)))
 		content_type = g_mime_content_type_new ("text", "plain");
 	
-	if (g_mime_content_type_is_type (content_type, "multipart", "*"))
+	parser_unstep (parser);
+	if (g_mime_content_type_is_type (content_type, "multipart", "*")) {
 		object = parser_construct_multipart (parser, content_type, found);
-	else
+	} else {
 		object = parser_construct_leaf_part (parser, content_type, found);
+	}
 	
 	message->mime_part = object;
 	
@@ -1384,10 +1237,11 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	struct _header_raw *header;
 	GMimeObject *object;
 	
-	g_assert (priv->state >= GMIME_PARSER_STATE_HEADERS_END);
+	/* get the headers */
+	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
+		;
 	
 	object = g_mime_object_new_type (content_type->type, content_type->subtype);
-	
 	header = priv->headers;
 	while (header) {
 		g_mime_object_add_header (object, header->name, header->value);
@@ -1403,27 +1257,23 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	g_mime_header_set_raw (object->headers, priv->rawbuf);
 	raw_header_reset (priv);
 	
-	if (priv->state == GMIME_PARSER_STATE_HEADERS_END) {
-		/* skip empty line after headers */
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
-			*found = FOUND_EOS;
-			return object;
-		}
-	}
+	/* skip empty line after headers */
+	parser_skip_line (parser);
 	
-	if (GMIME_IS_MESSAGE_PART (object))
+	if (GMIME_IS_MESSAGE_PART (object)) {
 		parser_scan_message_part (parser, (GMimeMessagePart *) object, found);
-	else
+	} else {
 		parser_scan_mime_part_content (parser, (GMimePart *) object, found);
+	}
 	
 	return object;
 }
 
 static void
-crlf2lf (char *in)
+crlf2lf (unsigned char *in)
 {
-	register char *inptr = in;
-	register char *outptr;
+	register unsigned char *inptr = in;
+	register unsigned char *outptr;
 	
 	while (*inptr != '\0' && !(inptr[0] == '\r' && inptr[1] == '\n'))
 		inptr++;
@@ -1448,8 +1298,8 @@ static int
 parser_scan_multipart_face (GMimeParser *parser, GMimeMultipart *multipart, gboolean preface)
 {
 	GByteArray *buffer;
-	int found, crlf;
-	char *face;
+	const char *face;
+	int crlf, found;
 	
 	buffer = g_byte_array_new ();
 	found = parser_scan_content (parser, buffer, &crlf);
@@ -1458,8 +1308,8 @@ parser_scan_multipart_face (GMimeParser *parser, GMimeMultipart *multipart, gboo
 		/* last '\n' belongs to the boundary */
 		g_byte_array_set_size (buffer, buffer->len + 1);
 		buffer->data[buffer->len - crlf - 1] = '\0';
-		face = (char *) buffer->data;
-		crlf2lf (face);
+		crlf2lf (buffer->data);
+		face = buffer->data;
 		
 		if (preface)
 			g_mime_multipart_set_preface (multipart, face);
@@ -1475,58 +1325,45 @@ parser_scan_multipart_face (GMimeParser *parser, GMimeMultipart *multipart, gboo
 #define parser_scan_multipart_preface(parser, multipart) parser_scan_multipart_face (parser, multipart, TRUE)
 #define parser_scan_multipart_postface(parser, multipart) parser_scan_multipart_face (parser, multipart, FALSE)
 
-static gboolean
-found_immediate_boundary (struct _GMimeParserPrivate *priv, gboolean end)
-{
-	struct _boundary_stack *s = priv->bounds;
-	size_t len = end ? s->boundarylenfinal : s->boundarylen;
-	
-	return !strncmp (priv->inptr, s->boundary, len)
-		&& (priv->inptr[len] == '\n' || priv->inptr[len] == '\r');
-}
-
 static int
 parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 {
-	struct _GMimeParserPrivate *priv = parser->priv;
 	GMimeContentType *content_type;
 	GMimeObject *subpart;
 	int found;
 	
 	do {
 		/* skip over the boundary marker */
-		if (parser_skip_line (parser) == -1) {
-			found = FOUND_EOS;
-			break;
-		}
+		parser_skip_line (parser);
 		
 		/* get the headers */
-		priv->state = GMIME_PARSER_STATE_HEADERS;
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
-			found = FOUND_EOS;
-			break;
-		}
-		
-		if (priv->state == GMIME_PARSER_STATE_COMPLETE && priv->headers == NULL) {
-			found = FOUND_END_BOUNDARY;
-			break;
-		}
+		parser_step_headers (parser);
 		
 		if (!(content_type = parser_content_type (parser)))
 			content_type = g_mime_content_type_new ("text", "plain");
 		
-		if (g_mime_content_type_is_type (content_type, "multipart", "*"))
+		parser_unstep (parser);
+		if (g_mime_content_type_is_type (content_type, "multipart", "*")) {
 			subpart = parser_construct_multipart (parser, content_type, &found);
-		else
+		} else {
 			subpart = parser_construct_leaf_part (parser, content_type, &found);
+		}
 		
 		g_mime_multipart_add_part (multipart, subpart);
 		g_object_unref (subpart);
-	} while (found == FOUND_BOUNDARY && found_immediate_boundary (priv, FALSE));
+	} while (found == FOUND_BOUNDARY);
 	
 	return found;
 }
 
+static gboolean
+found_immediate_boundary (struct _GMimeParserPrivate *priv)
+{
+	struct _boundary_stack *s = priv->bounds;
+	
+	return !strncmp (s->boundary, priv->inptr, s->boundarylenfinal);
+}
+
 static GMimeObject *
 parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type, int *found)
 {
@@ -1536,10 +1373,11 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 	const char *boundary;
 	GMimeObject *object;
 	
-	g_assert (priv->state >= GMIME_PARSER_STATE_HEADERS_END);
+	/* get the headers */
+	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
+		;
 	
 	object = g_mime_object_new_type (content_type->type, content_type->subtype);
-	
 	header = priv->headers;
 	while (header) {
 		g_mime_object_add_header (object, header->name, header->value);
@@ -1557,13 +1395,8 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 	
 	multipart = (GMimeMultipart *) object;
 	
-	if (priv->state == GMIME_PARSER_STATE_HEADERS_END) {
-		/* skip empty line after headers */
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
-			*found = FOUND_EOS;
-			return object;
-		}
-	}
+	/* skip empty line after headers */
+	parser_skip_line (parser);
 	
 	boundary = g_mime_content_type_get_parameter (content_type, "boundary");
 	if (boundary) {
@@ -1574,7 +1407,7 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 		if (*found == FOUND_BOUNDARY)
 			*found = parser_scan_multipart_subparts (parser, multipart);
 		
-		if (*found == FOUND_END_BOUNDARY && found_immediate_boundary (priv, TRUE)) {
+		if (*found == FOUND_END_BOUNDARY && found_immediate_boundary (priv)) {
 			/* eat end boundary */
 			parser_skip_line (parser);
 			parser_pop_boundary (parser);
@@ -1594,24 +1427,23 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 static GMimeObject *
 parser_construct_part (GMimeParser *parser)
 {
-	struct _GMimeParserPrivate *priv = parser->priv;
 	GMimeContentType *content_type;
 	GMimeObject *object;
 	int found;
 	
 	/* get the headers */
-	while (priv->state < GMIME_PARSER_STATE_HEADERS_END) {
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR)
-			return NULL;
-	}
+	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
+		;
 	
 	if (!(content_type = parser_content_type (parser)))
 		content_type = g_mime_content_type_new ("text", "plain");
 	
-	if (g_mime_content_type_is_type (content_type, "multipart", "*"))
+	parser_unstep (parser);
+	if (g_mime_content_type_is_type (content_type, "multipart", "*")) {
 		object = parser_construct_multipart (parser, content_type, &found);
-	else
+	} else {
 		object = parser_construct_leaf_part (parser, content_type, &found);
+	}
 	
 	return object;
 }
@@ -1638,34 +1470,25 @@ static GMimeMessage *
 parser_construct_message (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	unsigned long content_length = ULONG_MAX;
 	GMimeContentType *content_type;
 	struct _header_raw *header;
+	int content_length = -1;
 	GMimeMessage *message;
 	GMimeObject *object;
-	char *endptr;
-	int found;
+	int state, found;
 	
-	/* scan the from-line if we are parsing an mbox */
-	while (priv->state != GMIME_PARSER_STATE_HEADERS) {
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR)
-			return NULL;
-	}
+	/* get the headers (and, optionally, the from-line) */
+	while ((state = parser_step (parser)) != GMIME_PARSER_STATE_ERROR && state != GMIME_PARSER_STATE_HEADERS_END)
+		;
 	
-	/* parse the headers */
-	while (priv->state < GMIME_PARSER_STATE_HEADERS_END) {
-		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR)
-			return NULL;
-	}
+	if (state == GMIME_PARSER_STATE_ERROR)
+		return NULL;
 	
 	message = g_mime_message_new (FALSE);
 	header = priv->headers;
 	while (header) {
-		if (priv->respect_content_length && !g_ascii_strcasecmp (header->name, "Content-Length")) {
-			content_length = strtoul (header->value, &endptr, 10);
-			if (endptr == header->value)
-				content_length = ULONG_MAX;
-		}
+		if (priv->respect_content_length && !g_ascii_strcasecmp (header->name, "Content-Length"))
+			content_length = strtoul (header->value, NULL, 10);
 		
 		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
 		header = header->next;
@@ -1673,17 +1496,19 @@ parser_construct_message (GMimeParser *parser)
 	
 	if (priv->scan_from) {
 		parser_push_boundary (parser, "From ");
-		if (priv->respect_content_length && content_length < ULONG_MAX)
+		if (priv->respect_content_length && content_length != -1)
 			priv->bounds->content_end = parser_offset (priv, NULL) + content_length;
 	}
 	
 	if (!(content_type = parser_content_type (parser)))
 		content_type = g_mime_content_type_new ("text", "plain");
 	
-	if (content_type && g_mime_content_type_is_type (content_type, "multipart", "*"))
+	parser_unstep (parser);
+	if (content_type && g_mime_content_type_is_type (content_type, "multipart", "*")) {
 		object = parser_construct_multipart (parser, content_type, &found);
-	else
+	} else {
 		object = parser_construct_leaf_part (parser, content_type, &found);
+	}
 	
 	message->mime_part = object;
 	
@@ -1735,7 +1560,7 @@ g_mime_parser_get_from (GMimeParser *parser)
 		return NULL;
 	
 	if (priv->from_line->len)
-		return g_strndup ((char *) priv->from_line->data, priv->from_line->len);
+		return g_strndup (priv->from_line->data, priv->from_line->len);
 	
 	return NULL;
 }
@@ -1764,45 +1589,3 @@ g_mime_parser_get_from_offset (GMimeParser *parser)
 	
 	return priv->from_offset;
 }
-
-
-/**
- * g_mime_parser_get_headers_begin:
- * @parser: a #GMimeParser context
- *
- * Gets the stream offset of the beginning of the headers of the most
- * recently parsed message.
- *
- * Returns: the offset of the beginning of the headers of the most
- * recently parsed message or %-1 on error.
- *
- * Since: 2.2.23
- **/
-off_t
-g_mime_parser_get_headers_begin (GMimeParser *parser)
-{
-	g_return_val_if_fail (GMIME_IS_PARSER (parser), -1);
-	
-	return parser->priv->headers_begin;
-}
-
-
-/**
- * g_mime_parser_get_headers_end:
- * @parser: a #GMimeParser context
- *
- * Gets the stream offset of the end of the headers of the most
- * recently parsed message.
- *
- * Returns: the offset of the end of the headers of the most recently
- * parsed message or %-1 on error.
- *
- * Since: 2.2.23
- **/
-off_t
-g_mime_parser_get_headers_end (GMimeParser *parser)
-{
-	g_return_val_if_fail (GMIME_IS_PARSER (parser), -1);
-	
-	return parser->priv->headers_end;
-}
