@@ -1,21 +1,20 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public License
- *  as published by the Free Software Foundation; either version 2.1
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
- *  02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
 
@@ -68,7 +67,7 @@ g_mime_stream_fs_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeStreamFs),
-			0,    /* n_preallocs */
+			16,   /* n_preallocs */
 			(GInstanceInitFunc) g_mime_stream_fs_init,
 		};
 		
@@ -123,7 +122,7 @@ g_mime_stream_fs_finalize (GObject *object)
 static ssize_t
 stream_read (GMimeStream *stream, char *buf, size_t len)
 {
-	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
+	GMimeStreamFs *fstream = (GMimeStreamFs *) stream;
 	ssize_t nread;
 	
 	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
@@ -133,16 +132,16 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 		len = MIN (stream->bound_end - stream->position, (off_t) len);
 	
 	/* make sure we are at the right position */
-	lseek (fs->fd, stream->position, SEEK_SET);
+	lseek (fstream->fd, stream->position, SEEK_SET);
 	
 	do {
-		nread = read (fs->fd, buf, len);
+		nread = read (fstream->fd, buf, len);
 	} while (nread == -1 && errno == EINTR);
 	
 	if (nread > 0)
 		stream->position += nread;
 	else if (nread == 0)
-		fs->eos = TRUE;
+		fstream->eos = TRUE;
 	
 	return nread;
 }
@@ -150,9 +149,8 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 static ssize_t
 stream_write (GMimeStream *stream, const char *buf, size_t len)
 {
-	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
-	size_t nwritten = 0;
-	ssize_t n;
+	GMimeStreamFs *fstream = (GMimeStreamFs *) stream;
+	ssize_t nwritten = 0, n;
 	
 	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
 		return -1;
@@ -161,20 +159,17 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 		len = MIN (stream->bound_end - stream->position, (off_t) len);
 	
 	/* make sure we are at the right position */
-	lseek (fs->fd, stream->position, SEEK_SET);
+	lseek (fstream->fd, stream->position, SEEK_SET);
 	
 	do {
 		do {
-			n = write (fs->fd, buf + nwritten, len - nwritten);
+			n = write (fstream->fd, buf + nwritten, len - nwritten);
 		} while (n == -1 && (errno == EINTR || errno == EAGAIN));
 		
 		if (n > 0)
 			nwritten += n;
 	} while (n != -1 && nwritten < len);
 	
-	if (n == -1 && (errno == EFBIG || errno == ENOSPC))
-		fs->eos = TRUE;
-	
 	if (nwritten > 0)
 		stream->position += nwritten;
 	else if (n == -1)
@@ -186,71 +181,65 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 static int
 stream_flush (GMimeStream *stream)
 {
-	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
+	GMimeStreamFs *fstream = (GMimeStreamFs *) stream;
 	
-	g_return_val_if_fail (fs->fd != -1, -1);
+	g_return_val_if_fail (fstream->fd != -1, -1);
 	
-	return fsync (fs->fd);
+	return fsync (fstream->fd);
 }
 
 static int
 stream_close (GMimeStream *stream)
 {
-	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
-	int rv;
+	GMimeStreamFs *fstream = (GMimeStreamFs *) stream;
+	int ret;
 	
-	if (fs->fd == -1)
-		return 0;
+	g_return_val_if_fail (fstream->fd != -1, -1);
 	
-	do {
-		if ((rv = close (fs->fd)) == 0)
-			fs->fd = -1;
-	} while (rv == -1 && errno == EINTR);
+	ret = close (fstream->fd);
+	if (ret != -1)
+		fstream->fd = -1;
 	
-	return rv;
+	return ret;
 }
 
 static gboolean
 stream_eos (GMimeStream *stream)
 {
-	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
+	GMimeStreamFs *fstream = (GMimeStreamFs *) stream;
 	
-	g_return_val_if_fail (fs->fd != -1, TRUE);
+	g_return_val_if_fail (fstream->fd != -1, TRUE);
 	
-	return fs->eos;
+	return fstream->eos;
 }
 
 static int
 stream_reset (GMimeStream *stream)
 {
-	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
+	GMimeStreamFs *fstream = (GMimeStreamFs *) stream;
+	off_t ret;
 	
-	if (fs->fd == -1)
-		return -1;
+	g_return_val_if_fail (fstream->fd != -1, -1);
 	
-	if (stream->position == stream->bound_start) {
-		fs->eos = FALSE;
+	if (stream->position == stream->bound_start)
 		return 0;
+	
+	ret = lseek (fstream->fd, stream->bound_start, SEEK_SET);
+	if (ret != -1) {
+		fstream->eos = FALSE;
+		stream->position = stream->bound_start;
 	}
 	
-	/* FIXME: if stream_read/write is always going to lseek to
-	 * make sure fd's seek position matches our own, we could just
-	 * set stream->position = stream->bound_start and be done. */
-	if (lseek (fs->fd, stream->bound_start, SEEK_SET) == -1)
-		return -1;
-	
-	fs->eos = FALSE;
-	
-	return 0;
+	return ret;
 }
 
 static off_t
 stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 {
-	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
-	off_t real;
+	GMimeStreamFs *fstream = (GMimeStreamFs *) stream;
+	off_t real = stream->position;
 	
-	g_return_val_if_fail (fs->fd != -1, -1);
+	g_return_val_if_fail (fstream->fd != -1, -1);
 	
 	switch (whence) {
 	case GMIME_STREAM_SEEK_SET:
@@ -260,46 +249,29 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 		real = stream->position + offset;
 		break;
 	case GMIME_STREAM_SEEK_END:
-		if (offset > 0 || (stream->bound_end == -1 && !fs->eos)) {
-			/* need to do an actual lseek() here because
-			 * we either don't know the offset of the end
-			 * of the stream and/or don't know if we can
-			 * seek past the end */
-			if ((real = lseek (fs->fd, offset, SEEK_END)) == -1)
-				return -1;
-		} else if (fs->eos && stream->bound_end == -1) {
-			/* seeking backwards from eos (which happens
-			 * to be our current position) */
-			real = stream->position + offset;
-		} else {
-			/* seeking backwards from a known position */
-			real = stream->bound_end + offset;
+		if (stream->bound_end == -1) {
+			real = lseek (fstream->fd, offset, SEEK_END);
+			if (real != -1) {
+				if (real < stream->bound_start)
+					real = stream->bound_start;
+				stream->position = real;
+			}
+			return real;
 		}
-		
+		real = stream->bound_end + offset;
 		break;
-	default:
-		g_assert_not_reached ();
-		return -1;
 	}
 	
-	/* sanity check the resultant offset */
-	if (real < stream->bound_start)
+	if (stream->bound_end != -1)
+		real = MIN (real, stream->bound_end);
+	real = MAX (real, stream->bound_start);
+	
+	real = lseek (fstream->fd, real, SEEK_SET);
+	if (real == -1)
 		return -1;
 	
-	/* short-cut if we are seeking to our current position */
-	if (real == stream->position)
-		return real;
-	
-	if (stream->bound_end != -1 && real > stream->bound_end)
-		return -1;
-	
-	if ((real = lseek (fs->fd, real, SEEK_SET)) == -1)
-		return -1;
-	
-	/* reset eos if appropriate */
-	if ((stream->bound_end != -1 && real < stream->bound_end) ||
-	    (fs->eos && real < stream->position))
-		fs->eos = FALSE;
+	if (real != stream->position && fstream->eos)
+		fstream->eos = FALSE;
 	
 	stream->position = real;
 	
@@ -315,14 +287,14 @@ stream_tell (GMimeStream *stream)
 static ssize_t
 stream_length (GMimeStream *stream)
 {
-	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
+	GMimeStreamFs *fstream = (GMimeStreamFs *) stream;
 	off_t bound_end;
 	
-	if (stream->bound_end != -1)
+	if (stream->bound_start != -1 && stream->bound_end != -1)
 		return stream->bound_end - stream->bound_start;
 	
-	bound_end = lseek (fs->fd, 0, SEEK_END);
-	lseek (fs->fd, stream->position, SEEK_SET);
+	bound_end = lseek (fstream->fd, 0, SEEK_END);
+	lseek (fstream->fd, stream->position, SEEK_SET);
 	
 	if (bound_end < stream->bound_start)
 		return -1;
@@ -333,15 +305,15 @@ stream_length (GMimeStream *stream)
 static GMimeStream *
 stream_substream (GMimeStream *stream, off_t start, off_t end)
 {
-	GMimeStreamFs *fs;
+	GMimeStreamFs *fstream;
 	
-	fs = g_object_new (GMIME_TYPE_STREAM_FS, NULL);
-	g_mime_stream_construct (GMIME_STREAM (fs), start, end);
-	fs->fd = GMIME_STREAM_FS (stream)->fd;
-	fs->owner = FALSE;
-	fs->eos = FALSE;
+	fstream = g_object_new (GMIME_TYPE_STREAM_FS, NULL, NULL);
+	fstream->owner = FALSE;
+	fstream->fd = GMIME_STREAM_FS (stream)->fd;
 	
-	return (GMimeStream *) fs;
+	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
+	
+	return GMIME_STREAM (fstream);
 }
 
 
@@ -356,19 +328,19 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
 GMimeStream *
 g_mime_stream_fs_new (int fd)
 {
-	GMimeStreamFs *fs;
+	GMimeStreamFs *fstream;
 	off_t start;
 	
-	if ((start = lseek (fd, 0, SEEK_CUR)) == -1)
-		start = 0;
+	fstream = g_object_new (GMIME_TYPE_STREAM_FS, NULL, NULL);
+	fstream->owner = TRUE;
+	fstream->eos = FALSE;
+	fstream->fd = fd;
 	
-	fs = g_object_new (GMIME_TYPE_STREAM_FS, NULL);
-	g_mime_stream_construct (GMIME_STREAM (fs), start, -1);
-	fs->owner = TRUE;
-	fs->eos = FALSE;
-	fs->fd = fd;
+	start = lseek (fd, 0, SEEK_CUR);
 	
-	return (GMimeStream *) fs;
+	g_mime_stream_construct (GMIME_STREAM (fstream), start, -1);
+	
+	return GMIME_STREAM (fstream);
 }
 
 
@@ -386,15 +358,16 @@ g_mime_stream_fs_new (int fd)
 GMimeStream *
 g_mime_stream_fs_new_with_bounds (int fd, off_t start, off_t end)
 {
-	GMimeStreamFs *fs;
+	GMimeStreamFs *fstream;
 	
-	fs = g_object_new (GMIME_TYPE_STREAM_FS, NULL);
-	g_mime_stream_construct (GMIME_STREAM (fs), start, end);
-	fs->owner = TRUE;
-	fs->eos = FALSE;
-	fs->fd = fd;
+	fstream = g_object_new (GMIME_TYPE_STREAM_FS, NULL, NULL);
+	fstream->owner = TRUE;
+	fstream->eos = FALSE;
+	fstream->fd = fd;
 	
-	return (GMimeStream *) fs;
+	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
+	
+	return GMIME_STREAM (fstream);
 }
 
 
